@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 3910 2008-05-20 20:40:55Z joanied $"
-__version__   = "$Revision: 3910 $"
-__date__      = "$Date: 2008-05-20 16:40:55 -0400 (Tue, 20 May 2008) $"
+__id__        = "$Id: braillegenerator.py 4044 2008-07-18 12:45:13Z wwalker $"
+__version__   = "$Revision: 4044 $"
+__date__      = "$Date: 2008-07-18 08:45:13 -0400 (Fri, 18 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -122,6 +122,8 @@ class BrailleGenerator:
              self._getBrailleRegionsForRowHeader
         self.brailleGenerators[pyatspi.ROLE_SCROLL_BAR]          = \
              self._getBrailleRegionsForScrollBar
+        self.brailleGenerators[pyatspi.ROLE_SCROLL_PANE]         = \
+             self._getBrailleRegionsForScrollPane
         self.brailleGenerators[pyatspi.ROLE_SLIDER]              = \
              self._getBrailleRegionsForSlider
         self.brailleGenerators[pyatspi.ROLE_SPIN_BUTTON]         = \
@@ -192,6 +194,25 @@ class BrailleGenerator:
         else:
             return None
 
+    def _getTextForRequiredObject(self, obj):
+        """Returns a string to be displayed that describes the required
+        state of the given object.
+
+        Arguments:
+        - obj: the Accessible object
+
+        Returns a string to be displayed.
+        """
+
+        if not settings.presentRequiredState:
+            return None
+
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_REQUIRED):
+            return settings.brailleRequiredStateString
+        else:
+            return None
+
     def _getTextForRole(self, obj, role=None):
         if (settings.brailleVerbosityLevel \
             == settings.VERBOSITY_LEVEL_VERBOSE)\
@@ -243,6 +264,8 @@ class BrailleGenerator:
         text = self._script.appendString(text, 
                                          self._script.getTextForValue(obj))
         text = self._script.appendString(text, self._getTextForRole(obj, role))
+        text = self._script.appendString(text,
+                                         self._getTextForRequiredObject(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -340,6 +363,8 @@ class BrailleGenerator:
 
         text = self._script.appendString(
             text, self._getTextForRole(obj, pyatspi.ROLE_CHECK_BOX))
+        text = self._script.appendString(text,
+                                         self._getTextForRequiredObject(obj))
 
         regions = []
         componentRegion = braille.Component(
@@ -646,7 +671,7 @@ class BrailleGenerator:
 
         textRegion = braille.Text(obj,
                                   self._script.getDisplayedLabel(obj),
-                                  " $l")
+                                  settings.brailleEOLIndicator)
         regions.append(textRegion)
 
         # We do not want the role at the end of text areas.
@@ -832,8 +857,16 @@ class BrailleGenerator:
 
         textRegion = braille.Text(obj,
                                   self._script.getDisplayedLabel(obj),
-                                  " $l")
+                                  settings.brailleEOLIndicator)
         regions.append(textRegion)
+        text = self._getTextForRequiredObject(obj)
+        if text:
+            regions.append(braille.Region(" " + text))
+
+        if settings.presentReadOnlyText \
+           and not obj.getState().contains(pyatspi.STATE_EDITABLE):
+            regions.append(braille.Region(" " \
+                                          + settings.brailleReadOnlyString))
 
         # We do not want the role at the end of text areas.
 
@@ -1092,6 +1125,30 @@ class BrailleGenerator:
 
         return self._getDefaultBrailleRegions(obj)
 
+    def _getBrailleRegionsForScrollPane(self, obj):
+        """Get the braille for a scroll pane.
+
+        Arguments:
+        - obj: the scroll pane
+
+        Returns a list where the first element is a list of Regions to display
+        and the second element is the Region which should get focus.
+        """
+
+        self._debugGenerator("_getBrailleRegionsForScrollPane", obj)
+
+        # If this scroll pane is labelled by a page tab, then return the
+        # page tab information for the braille context instead. Thunderbird
+        # folder properties is such a case. See bug #507922 for more details.
+        #
+        relations = obj.getRelationSet()
+        for relation in relations:
+            if relation.getRelationType() ==  pyatspi.RELATION_LABELLED_BY:
+                labelledBy = relation.getTarget(0)
+                return self._getBrailleRegionsForPageTab(labelledBy)
+
+        return self._getDefaultBrailleRegions(obj)
+
     def _getBrailleRegionsForSlider(self, obj):
         """Get the braille for a slider.  If the object already
         had focus, just the value is displayed.
@@ -1116,6 +1173,8 @@ class BrailleGenerator:
         text = self._script.appendString(text,
                                          self._script.getTextForValue(obj))
         text = self._script.appendString(text, self._getTextForRole(obj))
+        text = self._script.appendString(text,
+                                         self._getTextForRequiredObject(obj))
 
         regions = []
         componentRegion = braille.Component(obj, text)
@@ -1351,8 +1410,8 @@ class BrailleGenerator:
             # the same row).
             #
             speakAll = True
-            if self._script.pointOfReference.has_key("lastRow") and \
-                self._script.pointOfReference.has_key("lastColumn"):
+            if "lastRow" in self._script.pointOfReference and \
+                "lastColumn" in self._script.pointOfReference:
                 pointOfReference = self._script.pointOfReference
                 speakAll = (pointOfReference["lastRow"] != row) or \
                        ((row == 0 or row == table.nRows-1) and \
@@ -1570,7 +1629,7 @@ class BrailleGenerator:
                     or (obj.getRole() == pyatspi.ROLE_PAGE_TAB_LIST)
 
         role = obj.getRole()
-        if self.brailleGenerators.has_key(role):
+        if role in self.brailleGenerators:
             generator = self.brailleGenerators[role]
         else:
             generator = self._getDefaultBrailleRegions
@@ -1693,7 +1752,22 @@ class BrailleGenerator:
             if (row >= 0) \
                 and (not obj.getRole() in [pyatspi.ROLE_ROW_HEADER,
                                            pyatspi.ROLE_TABLE_ROW_HEADER]):
+                # Get the header information.  In Java Swing, the
+                # information is not exposed via the description
+                # but is instead a header object, so we fall back
+                # to that if it exists.
+                #
+                # [[[TODO: WDW - the more correct thing to do, I 
+                # think, is to look at the row header object.
+                # We've been looking at the description for so 
+                # long, though, that we'll give the description 
+                # preference for now.]]]
+                #
                 desc = table.getRowDescription(row)
+                if not desc:
+                    header = table.getRowHeader(row)
+                    if header:
+                        desc = self._script.getDisplayedText(header)
             else:
                 desc = None
             if desc and len(desc):
@@ -1714,7 +1788,22 @@ class BrailleGenerator:
             if (col >= 0) \
                 and (not obj.getRole() in [pyatspi.ROLE_COLUMN_HEADER,
                                            pyatspi.ROLE_TABLE_COLUMN_HEADER]):
+                # Get the header information.  In Java Swing, the
+                # information is not exposed via the description
+                # but is instead a header object, so we fall back
+                # to that if it exists.
+                #
+                # [[[TODO: WDW - the more correct thing to do, I 
+                # think, is to look at the row header object.
+                # We've been looking at the description for so 
+                # long, though, that we'll give the description 
+                # preference for now.]]]
+                #
                 desc = table.getColumnDescription(col)
+                if not desc:
+                    header = table.getColumnHeader(col)
+                    if header:
+                        desc = self._script.getDisplayedText(header)
             else:
                 desc = None
             if desc and len(desc):
