@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id: flat_review.py 4054 2008-07-21 17:11:44Z joanied $"
+__version__   = "$Revision: 4054 $"
+__date__      = "$Date: 2008-07-21 13:11:44 -0400 (Mon, 21 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -569,14 +569,15 @@ class Line:
 
                 brailleOffset += len(region.string.decode("UTF-8"))
 
-            if len(self.brailleRegions):
-                pad = braille.Region(" ")
-                pad.brailleOffset = brailleOffset
-                self.brailleRegions.append(pad)
-                brailleOffset += 1
-            eol = braille.Region("$l")
-            eol.brailleOffset = brailleOffset
-            self.brailleRegions.append(eol)
+            if not settings.disableBrailleEOL:
+                if len(self.brailleRegions):
+                    pad = braille.Region(" ")
+                    pad.brailleOffset = brailleOffset
+                    self.brailleRegions.append(pad)
+                    brailleOffset += 1
+                eol = braille.Region("$l")
+                eol.brailleOffset = brailleOffset
+                self.brailleRegions.append(eol)
 
         return self.brailleRegions
 
@@ -1563,8 +1564,9 @@ class Context:
             # handle toolkits that will offset the caret position to
             # the right if you click dead on center of a character.
             #
+            x = max(x, x + (width / 2) - 1)
             eventsynthesizer.clickPoint(x,
-                                        y + height/ 2,
+                                        y + height / 2,
                                         button)
         except:
             debug.printException(debug.LEVEL_SEVERE)
