@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: script.py 3974 2008-06-11 21:48:33Z richb $"
-__version__   = "$Revision: 3974 $"
-__date__      = "$Date: 2008-06-11 17:48:33 -0400 (Wed, 11 Jun 2008) $"
+__id__        = "$Id: script.py 3996 2008-06-23 16:25:15Z richb $"
+__version__   = "$Revision: 3996 $"
+__date__      = "$Date: 2008-06-23 12:25:15 -0400 (Mon, 23 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1165,6 +1165,8 @@ class Script(default.Script):
     # 4) Calc: cell editor.
     # 5) Calc: name box.
     # 6) Calc: spreadsheet cell.
+    # 7) Impress: scroll pane.
+    # 8) Presentation: scroll pane: place holder
 
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
         """Called when the visual object with focus changes.
@@ -1444,6 +1446,54 @@ class Script(default.Script):
                     self.pointOfReference['lastRow'] = row
                 return
 
+        # 7) Impress: scroll pane.
+        #
+        # If we are in the slide presentation scroll pane, also announce
+        # the current page tab. See bug #538056 for more details.
+        #
+        rolesList = [pyatspi.ROLE_SCROLL_PANE, \
+                     pyatspi.ROLE_PANEL, \
+                     pyatspi.ROLE_PANEL, \
+                     pyatspi.ROLE_ROOT_PANE, \
+                     pyatspi.ROLE_FRAME, \
+                     pyatspi.ROLE_APPLICATION]
+
+        if self.isDesiredFocusedItem(event.source, rolesList):
+            debug.println(self.debugLevel, "soffice.locusOfFocusChanged - " \
+                          + "Impress: scroll pane.")
+
+            for child in event.source.parent:
+                if child.getRole() == pyatspi.ROLE_PAGE_TAB_LIST:
+                    for tab in child:
+                        eventState = tab.getState()
+                        if eventState.contains(pyatspi.STATE_SELECTED):
+                            utterances = self.speechGenerator.getSpeech(tab,
+                                                                        False)
+                            speech.speakUtterances(utterances)
+            # Fall-thru to process the event with the default handler.
+
+        # 8) Presentation: scroll pane: place holder
+        #
+        # If we are focused on a place holder element in the slide
+        # presentation scroll pane, first present the object, then
+        # try to present each of its children. See bug #538064 for
+        # more details.
+        #
+        rolesList = [pyatspi.ROLE_UNKNOWN,
+                     pyatspi.ROLE_UNKNOWN,
+                     pyatspi.ROLE_SCROLL_PANE, \
+                     pyatspi.ROLE_PANEL, \
+                     pyatspi.ROLE_PANEL, \
+                     pyatspi.ROLE_ROOT_PANE, \
+                     pyatspi.ROLE_FRAME, \
+                     pyatspi.ROLE_APPLICATION]
+        if self.isDesiredFocusedItem(event.source, rolesList):
+            default.Script.locusOfFocusChanged(self, event,
+                                    oldLocusOfFocus, newLocusOfFocus)
+            for child in event.source:
+                speech.speak(self.getText(child, 0, -1), None, False)
+            return
+
         # Pass the event onto the parent class to be handled in the default way.
 
         default.Script.locusOfFocusChanged(self, event,
@@ -1698,17 +1748,23 @@ class Script(default.Script):
             return
 
         # Two events are received when the caret moves
-        # to a new paragraph. The first is a focus event
+        # to a new paragraph in oowriter. The first is a focus event
         # (in the form of object:state-changed:focused
         # instead of focus:). The second is a caret-moved
         # event. Just set the locusOfFocus for the first event.
         #
-        if event.type.startswith("object:state-changed:focused") and \
-           event.source.getRole() == pyatspi.ROLE_PARAGRAPH and \
-           event.source != self.currentParagraph:
-            self.currentParagraph = event.source
-            orca.setLocusOfFocus(event, event.source, False)
-            return
+        if event.type.startswith("object:state-changed:focused"):
+            rolesList = [pyatspi.ROLE_PARAGRAPH, \
+                         pyatspi.ROLE_UNKNOWN, \
+                         pyatspi.ROLE_SCROLL_PANE, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_ROOT_PANE, \
+                         pyatspi.ROLE_FRAME]
+            if self.isDesiredFocusedItem(event.source, rolesList) and \
+               event.source != self.currentParagraph:
+                self.currentParagraph = event.source
+                orca.setLocusOfFocus(event, event.source, False)
+                return
 
         # If we are in the sbase Table Wizard, try to reduce the numerous
         # utterances of "Available fields panel". See bug #465087 for
@@ -1852,7 +1908,49 @@ class Script(default.Script):
         - event: the Event
         """
 
-        # If the object is losing focus, then just ignore this event.
+        # If we are losing focus and we in:
+        # 1/ a paragraph in an ooimpress slide presentation
+        # 2/ a paragraph in an oowriter text document
+        # and the last thing the user typed was a Return, and echo by word
+        # is enabled, and the last focused object was not of role "unknown",
+        # then echo the previous word that the user typed.
+        # See bug #538053 and bug #538835 for more details.
+        #
+        if event.detail1 == -1:
+            # ooimpress paragraph in a slide presentation.
+            rolesList = [pyatspi.ROLE_PARAGRAPH,
+                         pyatspi.ROLE_UNKNOWN,
+                         pyatspi.ROLE_UNKNOWN,
+                         pyatspi.ROLE_SCROLL_PANE, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_PANEL, \
+                         pyatspi.ROLE_ROOT_PANE, \
+                         pyatspi.ROLE_FRAME, \
+                         pyatspi.ROLE_APPLICATION]
+
+            # oowriter paragraph in a text document.
+            rolesList1 = [pyatspi.ROLE_PARAGRAPH,
+                          pyatspi.ROLE_UNKNOWN,
+                          pyatspi.ROLE_SCROLL_PANE, \
+                          pyatspi.ROLE_PANEL, \
+                          pyatspi.ROLE_ROOT_PANE, \
+                          pyatspi.ROLE_FRAME, \
+                          pyatspi.ROLE_APPLICATION]
+            if settings.enableEchoByWord and \
+               (self.isDesiredFocusedItem(event.source, rolesList) or
+                self.isDesiredFocusedItem(event.source, rolesList1)):
+                if isinstance(orca_state.lastInputEvent,
+                              input_event.KeyboardEvent):
+                    keyString = orca_state.lastNonModifierKeyEvent.event_string
+                    focusRole = orca_state.locusOfFocus.getRole()
+                    if focusRole != pyatspi.ROLE_UNKNOWN and \
+                       keyString == "Return":
+                        result = self.getText(event.source, 0, -1)
+                        line = result.decode("UTF-8")
+                        self.echoPreviousWord(event.source, len(line))
+                        return
+
+        # Otherwise, if the object is losing focus, then just ignore this event.
         #
         if event.detail1 == -1:
             return
