@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3957 2008-06-04 16:43:04Z richb $"
-__version__   = "$Revision: 3957 $"
-__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
+__id__        = "$Id: speechgenerator.py 4044 2008-07-18 12:45:13Z wwalker $"
+__version__   = "$Revision: 4044 $"
+__date__      = "$Date: 2008-07-18 08:45:13 -0400 (Fri, 18 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -92,6 +92,8 @@ class SpeechGenerator:
              self._getSpeechForLayeredPane
         self.speechGenerators[pyatspi.ROLE_LIST]                = \
              self._getSpeechForList
+        self.speechGenerators[pyatspi.ROLE_LIST_ITEM]           = \
+             self._getSpeechForListItem
         self.speechGenerators[pyatspi.ROLE_MENU]                = \
              self._getSpeechForMenu
         self.speechGenerators[pyatspi.ROLE_MENU_BAR]            = \
@@ -239,6 +241,25 @@ class SpeechGenerator:
 
         return utterance
 
+    def _getSpeechForRequiredObject(self, obj):
+        """Returns the list of utterances that describe the required state
+        of the given object.
+
+        Arguments:
+        - obj: the Accessible object
+
+        Returns a list of utterances to be spoken.
+        """
+
+        if not settings.presentRequiredState:
+            return []
+
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_REQUIRED):
+            return [settings.speechRequiredStateString]
+        else:
+            return []
+
     def _debugGenerator(self, generatorName, obj, already_focused, utterances):
         """Prints debug.LEVEL_FINER information regarding the speech generator.
 
@@ -423,6 +444,7 @@ class SpeechGenerator:
                 utterances.extend(self._getSpeechForObjectRole(obj))
             utterances.append(checkedState)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self._getSpeechForRequiredObject(obj))
         else:
             utterances.append(checkedState)
 
@@ -806,7 +828,16 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken for the object.
         """
 
-        utterances = self._getDefaultSpeech(obj, already_focused)
+        utterances = []
+ 
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForAllTextSelection(obj))
+        utterances.extend(self._getSpeechForObjectAvailability(obj))
 
         # If already in focus then the tree probably collapsed or expanded
         state = obj.getState()
@@ -929,6 +960,11 @@ class SpeechGenerator:
         if len(utterances) == 0:
             if obj.name and (len(obj.name)):
                 utterances.append(obj.name)
+
+        if settings.presentReadOnlyText \
+           and not obj.getState().contains(pyatspi.STATE_EDITABLE):
+            utterances.append(settings.speechReadOnlyString)
+
         if obj.getRole() != pyatspi.ROLE_PARAGRAPH:
             utterances.extend(self._getSpeechForObjectRole(obj))
 
@@ -1210,6 +1246,7 @@ class SpeechGenerator:
             utterances.extend(self._getSpeechForObjectRole(obj))
             utterances.append(valueString)
             utterances.extend(self._getSpeechForObjectAvailability(obj))
+            utterances.extend(self._getSpeechForRequiredObject(obj))
 
         self._debugGenerator("_getSpeechForSlider",
                              obj,
@@ -1233,6 +1270,7 @@ class SpeechGenerator:
             utterances = [self._script.getDisplayedText(obj)]
         else:
             utterances = self._getDefaultSpeech(obj, already_focused)
+            utterances.extend(self._getSpeechForRequiredObject(obj))
 
         self._debugGenerator("_getSpeechForSpinButton",
                              obj,
@@ -1428,6 +1466,8 @@ class SpeechGenerator:
                 #
                 utterances.append(_("collapsed"))
 
+        utterances.extend(self._getSpeechForRequiredObject(obj))
+
         self._debugGenerator("_getSpeechForTableCell",
                              obj,
                              already_focused,
