@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 3957 2008-06-04 16:43:04Z richb $"
-__version__   = "$Revision: 3957 $"
-__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
+__id__        = "$Id: where_am_I.py 4055 2008-07-21 17:53:36Z wwalker $"
+__version__   = "$Revision: 4055 $"
+__date__      = "$Date: 2008-07-21 13:53:36 -0400 (Mon, 21 Jul 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -31,6 +31,7 @@ import orca_state
 import rolenames
 import settings
 import speech
+import text_attribute_names
 import urlparse, urllib2
 
 from orca_i18n import _ # for gettext support
@@ -117,7 +118,8 @@ class WhereAmI:
         elif role == pyatspi.ROLE_PAGE_TAB:
             self._speakPageTab(obj, basicOnly)
 
-        elif role in [pyatspi.ROLE_TEXT,
+        elif role in [pyatspi.ROLE_ENTRY,
+                      pyatspi.ROLE_TEXT,
                       pyatspi.ROLE_TERMINAL]:
             self._speakText(obj, basicOnly)
 
@@ -177,6 +179,10 @@ class WhereAmI:
         accelerator = self._getObjAccelerator(obj)
         utterances.append(accelerator)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "check box utterances=%s" \
                       % utterances)
         speech.speakUtterances(utterances)
@@ -197,6 +203,10 @@ class WhereAmI:
         text = self._getGroupLabel(obj)
         utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         text = self.getObjLabelAndName(obj) + " " + \
                rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
@@ -273,20 +283,25 @@ class WhereAmI:
         """
 
         utterances = []
-        text = self._getObjLabel(obj)
-        utterances.append(text)
+        label = self._getObjLabel(obj)
+        utterances.append(label)
 
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
         name = self._getObjName(obj)
-        utterances.append(name)
+        if name != label:
+            utterances.append(name)
         
         utterances.extend(self._getSpeechForAllTextSelection(obj))
 
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "spin button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -341,6 +356,10 @@ class WhereAmI:
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "slider utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -466,13 +485,25 @@ class WhereAmI:
         cell = self._script.getAncestor(obj,
                                         [pyatspi.ROLE_TABLE_CELL],
                                         [pyatspi.ROLE_FRAME])
-        if cell:
-            return self._speakTableCell(cell, basicOnly)
+        if cell and not self._script.isLayoutOnly(cell.parent):
+            # [[[TODO: WDW - we handle ROLE_ENTRY specially here because
+            # there is a bug in getRealActiveDescendant: it doesn't dive
+            # deep enough into the hierarchy (see comment #12 of bug
+            # #542714).  So, we'll do this nasty hack until we can feel
+            # more comfortable with mucking around with
+            # getRealActiveDescendant.]]]
+            #
+            if obj.getRole() != pyatspi.ROLE_ENTRY:
+                return self._speakTableCell(cell, basicOnly)
 
         utterances = []
         text = self._getObjLabel(obj)
         utterances.append(text)
 
+        if settings.presentReadOnlyText \
+           and not obj.getState().contains(pyatspi.STATE_EDITABLE):
+            utterances.append(settings.speechReadOnlyString)
+
         text = rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
@@ -565,6 +596,10 @@ class WhereAmI:
             text = self._getTableCell(obj)
             utterances.append(text)
 
+        text = self._getRequiredState(obj)
+        if text:
+            utterances.append(text)
+
         debug.println(self._debugLevel, "first table cell utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -981,7 +1016,7 @@ class WhereAmI:
             name = obj.description
 
         if name and name != "None":
-            text = name
+            text = name.strip()
         debug.println(self._debugLevel, "%s name=<%s>" % (obj.getRole(), text))
 
         return text
@@ -994,7 +1029,7 @@ class WhereAmI:
         label = self._script.getDisplayedLabel(obj)
 
         if label and label != "None":
-            text = label
+            text = label.strip()
 
         debug.println(self._debugLevel, "%s label=<%s>" % (obj.getRole(), text))
 
@@ -1536,8 +1571,11 @@ class WhereAmI:
                           "charDict: %s" % (charDict))
 
             for key in keys:
+                localizedKey = text_attribute_names.getTextAttributeName(key)
                 if key in charDict:
                     attribute = charDict[key]
+                    localizedValue = \
+                        text_attribute_names.getTextAttributeName(attribute)
                     if attribute:
                         # If it's the 'weight' attribute and greater than 400,
                         # just speak it as bold, otherwise speak the weight.
@@ -1551,15 +1589,15 @@ class WhereAmI:
                         elif key == "underline":
                             if attribute != "none":
                                 attribStr += " "
-                                attribStr += key
+                                attribStr += localizedKey
 
                         elif key == "style":
                             if attribute != "normal":
                                 attribStr += " "
-                                attribStr += attribute
+                                attribStr += localizedValue
                         else:
                             attribStr += " "
-                            attribStr += (key + " " + attribute)
+                            attribStr += (localizedKey + " " + localizedValue)
 
             # Also check to see if this is a hypertext link.
             #
@@ -1814,3 +1852,23 @@ class WhereAmI:
         debug.println(self._debugLevel, "toolbar utterances=%s" \
                       % utterances)
         speech.speakUtterances(utterances)
+
+    def _getRequiredState(self, obj):
+        """Returns a string describing the required state of the given
+        object.
+
+        Arguments:
+        - obj: the Accessible object
+        """
+
+        if not settings.presentRequiredState:
+            return ""
+
+        if obj.getRole() == pyatspi.ROLE_RADIO_BUTTON:
+            obj = obj.parent
+
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_REQUIRED):
+            return settings.speechRequiredStateString
+        else:
+            return ""
